import { AppTask, IAttrMapper, IContainer, NodeObserverLocator, Registration } from 'aurelia';
import { MdcSelect } from './mdc-select';
import { MdcSelectIcon } from './mdc-select-icon';
import { MdcSelectHelperText } from './mdc-select-helper-text/mdc-select-helper-text';
import { FloatingLabelConfiguration } from '@aurelia2-mdc-web/floating-label';
import { LineRippleConfiguration } from '@aurelia2-mdc-web/line-ripple';
import { ListConfiguration } from '@aurelia2-mdc-web/list';
import { MenuConfiguration } from '@aurelia2-mdc-web/menu';
import { NotchedOutlineConfiguration } from '@aurelia2-mdc-web/notched-outline';
import { RippleConfiguration } from '@aurelia2-mdc-web/ripple';
import { IMdcDefaultSelectConfiguration, defaultOptions } from './mdc-default-select-configuration';
export { MdcSelect } from './mdc-select';
function configure(container, config = defaultOptions) {
    const twoWay = AppTask.creating(IContainer, (container) => {
        const attrMapper = container.get(IAttrMapper);
        const nodeObserverLocator = container.get(NodeObserverLocator);
        attrMapper.useTwoWay((el, property) => {
            if (el.tagName === "MDC-SELECT") {
                return property === "value";
            }
            return false;
        });
        const valuePropertyConfig = { events: ['input', 'change'] };
        nodeObserverLocator.useConfig({
            'MDC-SELECT': {
                value: valuePropertyConfig
            },
        });
    });
    return container.register(twoWay, MdcSelect, MdcSelectIcon, MdcSelectHelperText, FloatingLabelConfiguration, LineRippleConfiguration, ListConfiguration, MenuConfiguration, NotchedOutlineConfiguration, RippleConfiguration, AppTask.hydrating(IContainer, async (container) => {
        Registration.instance(IMdcDefaultSelectConfiguration, config).register(container);
    }));
}
export const SelectConfiguration = {
    register(container) {
        return configure(container);
    },
    customize(config) {
        return {
            register(container) {
                return configure(container, config);
            },
        };
    }
};
//# sourceMappingURL=index.js.map